package com.mortals.xhx.module.customer.service.impl;

import cn.hutool.core.util.PhoneUtil;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.SecurityUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.CustomerSatusEnum;
import com.mortals.xhx.common.code.UserStatus;
import com.mortals.xhx.module.customer.dao.CustomerDao;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.model.CustomerEntityExt;
import com.mortals.xhx.module.customer.model.CustomerQuery;
import com.mortals.xhx.module.customer.model.vo.CustomerCensusVo;
import com.mortals.xhx.module.customer.model.vo.CustomerDesignCensusVo;
import com.mortals.xhx.module.customer.service.CustomerService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.*;

/**
* CustomerService
* 客户管理 service实现
*
* @author zxfei
* @date 2022-06-08
*/
@Service("customerService")
public class CustomerServiceImpl extends AbstractCRUDServiceImpl<CustomerDao, CustomerEntity, Long> implements CustomerService {

    @Override
    protected void saveBefore(CustomerEntity entity, Context context) throws AppException {
        this.validData(entity, context);
        if(StringUtils.isEmpty(entity.getContactTelphone())){
            throw new AppException("手机号不能为空");
        }
        if(StringUtils.isEmpty(entity.getCustName())){
            throw new AppException("姓名不能为空");
        }
        if(!PhoneUtil.isPhone(entity.getContactTelphone())){
            throw new AppException("手机号码格式不正确");
        }
        CustomerEntity query = new CustomerEntity();
        query.setContactTelphone(entity.getContactTelphone());
        List<CustomerEntity> list = dao.getList(query);
        if(CollectionUtils.isNotEmpty(list)){
            throw new AppException("手机号码为:"+entity.getContactTelphone()+"的用户已存在");
        }
        if (StringUtils.isNotEmpty(entity.getPassword())) {
            try {
                entity.setPassword(SecurityUtil.md5DoubleEncoding(entity.getPassword()));
            } catch (Exception e) {
                throw new AppException("密码转换异常");
            }
        } else {
            entity.setPassword(null);
        }
    }

    @Override
    protected CustomerEntity findBefore(CustomerEntity entity, PageInfo pageInfo, Context context) throws AppException {
        CustomerQuery query = new CustomerQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            CustomerQuery condition1 = new CustomerQuery();
            CustomerQuery condition2 = new CustomerQuery();
            CustomerQuery condition3 = new CustomerQuery();
            condition1.setContactTelphone(condition.toString());
            condition2.setLoginName(condition.toString());
            condition3.setCustName(condition.toString());
            List<CustomerQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            orConditionList.add(condition3);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    public Result<CustomerEntityExt> findExt(CustomerEntity params, PageInfo pageInfo, Context context) throws AppException {
        CustomerEntity query = this.findBefore(params, pageInfo, context);
        Result<CustomerEntityExt> result = getDao().getListExt(query,pageInfo);
        List<CustomerEntityExt> list = result.getList();
        if(CollectionUtils.isNotEmpty(list)){
            list.stream().forEach(item->{
                item.setCreateTimeStr(DateUtils.getDateTime(item.getCreateTime(),DateUtils.P_yyyy_MM_dd_HH_mm_ss));
            });
        }
        return result;
    }

    @Override
    public CustomerEntity doLogin(String loginName, String password, String loginIp) throws AppException {
        CustomerEntity user = this.findByLoginName(loginName);
        if (user == null || !user.getLoginName().equals(loginName)) {
            throw new AppException("用户名不存在！");
        }
        if(user.getStatus()!= CustomerSatusEnum.NORMAL.getValue()){
            throw new AppException("该账号已被禁用！");
        }
        try {
            if (!user.getPassword().equals(SecurityUtil.md5DoubleEncoding(password))) {
                throw new AppException("登录密码错误！");
            }
        } catch (Exception e) {
            throw new AppException("密码验认出错！", e);
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(user.getId());
        update.setLastLoginAddress(loginIp);
        update.setLastLoginTime(new Date());
        this.update(update);
        return user;
    }

    private CustomerEntity findByLoginName(String loginName) {
        CustomerEntity params = new CustomerEntity();
        params.setLoginName(loginName);
        return this.selectOne(params);
    }

    @Override
    public CustomerEntity updateByPersonal(CustomerEntity params, Context context) throws AppException {
        CustomerEntity update = new CustomerEntity();
        update.setId(params.getId());
        update.setLoginName(params.getLoginName());
        update.setCustName(params.getCustName());
        update.setAvatar(params.getAvatar());
        update.setSex(params.getSex());
        update.setOrganization(params.getOrganization());
        update.setContactTelphone(params.getContactTelphone());
        update.setMailbox(params.getMailbox());
        update.setJob(params.getJob());
        int iRet = this.dao.update(update);
        if (iRet == 0) {
            throw new AppException(-1002, "更新失败！");
        } else {
            params.setPassword(null);
            return params;
        }
    }

    @Override
    public CustomerEntity changePassword(CustomerEntity params, Context context) throws AppException {
        if(StringUtils.isEmpty(params.getOldPassword())||StringUtils.isEmpty(params.getNewPassword())){
            throw new AppException("密码不能为空");
        }
        if(params.getOldPassword().equals(params.getNewPassword())){
            throw new AppException("新密码不能与旧密码一致");
        }
        CustomerEntity old = this.get(params.getId());
        if(old==null){
            throw new AppException("客户信息不存在");
        }
        String newPwd = "";
        String oldPwd = "";
        try {
            oldPwd = SecurityUtil.md5DoubleEncoding(params.getOldPassword());
            newPwd = SecurityUtil.md5DoubleEncoding(params.getNewPassword());
        }catch (Exception e) {
            throw new AppException("密码转换异常");
        }
        if(old.getPassword().equals(oldPwd)){
            CustomerEntity update = new CustomerEntity();
            update.setId(params.getId());
            update.setPassword(newPwd);
            this.update(update,context);
            old.setPassword(null);
        }else {
            throw new AppException("原来的密码不正确");
        }
        return old;
    }

    @Override
    public void changePasswordByAdmin(CustomerEntity params, Context context) throws AppException {
        CustomerEntity old = this.get(params.getId());
        if(old==null){
            throw new AppException("客户信息不存在");
        }
        String newPwd = "";
        String oldPwd = "";
        try {
            newPwd = SecurityUtil.md5DoubleEncoding(params.getNewPassword());
        }catch (Exception e) {
            throw new AppException("密码转换异常");
        }
        CustomerEntity update = new CustomerEntity();
        update.setId(params.getId());
        update.setPassword(newPwd);
        this.update(update,context);
    }

    @Override
    public Map<String, Object> getCustomerCensus() {
        Map<String, Object> result = new HashMap<>();
        List<CustomerCensusVo> customerCensusVos = dao.getCensus();
        CustomerDesignCensusVo designCensusVo = dao.getDesignCensus();
        if(CollectionUtils.isNotEmpty(customerCensusVos)){
            for(CustomerCensusVo item:customerCensusVos){
                if(item.getMemberLevel()==1){
                    result.put("trial",item.getCustomerCount());
                }
                if(item.getMemberLevel()==2){
                    result.put("vip",item.getCustomerCount());
                }
                if(item.getMemberLevel()==3){
                    result.put("designer",item.getCustomerCount());
                }
            }
        }
        result.put("picturesCount",designCensusVo.getPicturesCount());
        result.put("videosCount",designCensusVo.getVideosCount());
        return result;
    }
}