package com.mortals.xhx.module.customer.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.module.customer.dao.CustomerWorkCollectDao;
import com.mortals.xhx.module.customer.model.CustomerWorkCollectEntity;
import com.mortals.xhx.module.customer.model.CustomerWorkDesignEntity;
import com.mortals.xhx.module.customer.service.CustomerWorkCollectService;
import com.mortals.xhx.module.design.model.DesignMasterplateEntity;
import com.mortals.xhx.module.design.service.DesignMasterplateService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
* CustomerWorkCollectService
* 客户收藏信息 service实现
*
* @author zxfei
* @date 2022-06-13
*/
@Service("customerWorkCollectService")
public class CustomerWorkCollectServiceImpl extends AbstractCRUDServiceImpl<CustomerWorkCollectDao, CustomerWorkCollectEntity, Long> implements CustomerWorkCollectService {

    @Autowired
    private DesignMasterplateService designMasterplateService;

    @Override
    protected CustomerWorkCollectEntity findBefore(CustomerWorkCollectEntity params, PageInfo pageInfo, Context context) throws AppException {
        pageInfo.setPrePageResult(-1);
        return super.findBefore(params, pageInfo, context);
    }

    @Override
    protected void findAfter(CustomerWorkCollectEntity params, PageInfo pageInfo, Context context, List<CustomerWorkCollectEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            list.stream().forEach(item->{
                DesignMasterplateEntity designMasterplateEntity = designMasterplateService.get(item.getMasterplateId());
                if(designMasterplateEntity!=null && StringUtils.isNotEmpty(designMasterplateEntity.getDraft())){
                    designMasterplateEntity.setDesignContent(JSONObject.parseArray(designMasterplateEntity.getDraft(),JSONObject.class));
                }
                item.setDesignMasterplate(designMasterplateEntity!=null?designMasterplateEntity:null);
            });
        }
    }

    @Override
    protected void saveBefore(CustomerWorkCollectEntity entity, Context context) throws AppException {
        this.validData(entity, context);
        entity.setCustomerId(this.getContextUserId(context));
        CustomerWorkCollectEntity query = new CustomerWorkCollectEntity();
        query.setCustomerId(entity.getCustomerId());
        query.setMasterplateId(entity.getMasterplateId());
        List<CustomerWorkCollectEntity> list = this.find(query);
        if(CollectionUtils.isNotEmpty(list)){
            throw new AppException("已收藏此模板，请勿重复收藏");
        }
    }

    @Override
    public int removeByMasterplateId(Long masterplateId, Context context) {
        CustomerWorkCollectEntity condition = new CustomerWorkCollectEntity();
        condition.setMasterplateId(masterplateId);
        condition.setCustomerId(this.getContextUserId(context));
        return this.dao.delete(condition);
    }
}