package com.mortals.xhx.module.customer.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.IBaseEnum;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.utils.ReflectUtils;
import com.mortals.framework.utils.poi.ExcelUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.CustomerSatusEnum;
import com.mortals.xhx.common.code.CustomerSrcEnum;
import com.mortals.xhx.common.code.MemberLevelEnum;
import com.mortals.xhx.common.code.SexEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.model.CustomerEntityExt;
import com.mortals.xhx.module.customer.model.CustomerQuery;
import com.mortals.xhx.module.customer.service.CustomerService;
import org.springframework.web.bind.annotation.*;

import java.util.*;
/**
*
* 客户管理
*
* @author zxfei
* @date 2022-06-08
*/
@RestController
@RequestMapping("customer")
public class CustomerController extends BaseCRUDJsonBodyMappingController<CustomerService,CustomerEntity,Long> {

    public CustomerController(){
        super.setModuleDesc( "客户管理");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model,"memberLevel", IBaseEnum.getEnumMap(MemberLevelEnum.class));
        this.addDict(model,"sex", IBaseEnum.getEnumMap(SexEnum.class));
        this.addDict(model,"status", IBaseEnum.getEnumMap(CustomerSatusEnum.class));
        this.addDict(model,"customerSrc", IBaseEnum.getEnumMap(CustomerSrcEnum.class));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(CustomerEntity query, Map<String, Object> model, Context context) throws AppException {
        Map<String,String> orderCols = new HashMap<>();
        orderCols.put("createTime","DESC");
        query.setOrderCols(orderCols);
    }

    @Override
    @UnAuth
    public Rest<Object> list(@RequestBody(required = false) CustomerEntity query) {
        Map<String, Object> model = new HashMap();
        Rest<Object> ret = new Rest();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();

        int code;
        try {
            PageInfo pageInfo = this.buildPageInfo(query);
            this.doListBefore(query, model, context);
            Result result = this.getService().findExt(query, pageInfo, context);
            Map<String,Object> census = service.getCustomerCensus();
            model.put("data", result.getList());
            model.put("pageInfo", result.getPageInfo());
            model.put("trial", census.get("trial"));
            model.put("vip", census.get("vip"));
            model.put("designer", census.get("designer"));
            model.put("picturesCount", census.get("picturesCount"));
            model.put("videosCount", census.get("videosCount"));
            this.parsePageInfo(model, result.getPageInfo());
            code = this.doListAfter(query, model, context);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict") == null ? null : (Map)model.remove("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    @GetMapping({"status/enable"})
    public String enableStatus(Long[] id) {
        Context context = this.getContext();
        if (id == null) {
            return this.createFailJsonResp("请至少选择一条记录");
        } else {
            Map<String, Object> model = new HashMap();
            int code = 1;
            String busiDesc = "启用客户" + this.getModuleDesc();

            try {
                CustomerEntity entity = new CustomerEntity();
                entity.setStatus(CustomerSatusEnum.NORMAL.getValue());
                CustomerQuery condition = new CustomerQuery();
                List<Long> idList = new ArrayList<>(id.length);
                Collections.addAll(idList,id);
                condition.setIdList(idList);
                int iRet = this.service.updateBatch(entity,condition,context);
                model.put("message_info", this.getModuleDesc() + "启用成功!启用条数:" + iRet);
                this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + StringUtils.converArray2Str(id) + "]");
            } catch (Exception var7) {
                code = -1;
                this.doException(this.request, busiDesc, model, var7);
            }

            JSONObject ret = new JSONObject();
            ret.put("code", Integer.valueOf(code));
            ret.put("msg", model.remove("message_info"));
            ret.put("data", model);
            return ret.toJSONString();
        }
    }

    @GetMapping({"status/disable"})
    public String disableStatus(Long[] id) {
        Context context = this.getContext();
        if (id == null) {
            return this.createFailJsonResp("请至少选择一条记录");
        } else {
            Map<String, Object> model = new HashMap();
            int code = 1;
            String busiDesc = "禁用客户" + this.getModuleDesc();

            try {
                CustomerEntity entity = new CustomerEntity();
                entity.setStatus(CustomerSatusEnum.DISABLE.getValue());
                CustomerQuery condition = new CustomerQuery();
                List<Long> idList = new ArrayList<>(id.length);
                Collections.addAll(idList,id);
                condition.setIdList(idList);
                int iRet = this.service.updateBatch(entity,condition,context);
                model.put("message_info", this.getModuleDesc() + "禁用成功!禁用条数:" + iRet);
                this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + StringUtils.converArray2Str(id) + "]");
            } catch (Exception var7) {
                code = -1;
                this.doException(this.request, busiDesc, model, var7);
            }

            JSONObject ret = new JSONObject();
            ret.put("code", Integer.valueOf(code));
            ret.put("msg", model.remove("message_info"));
            ret.put("data", model);
            return ret.toJSONString();
        }
    }

    @Override
    protected int editAfter(Long id, Map<String, Object> model, CustomerEntity entity, Context context) throws AppException {
        entity.setPassword(null);
        return 1;
    }

    @Override
    protected int viewAfter(Long id, Map<String, Object> model, CustomerEntity entity, Context context) throws AppException {
        entity.setPassword(null);
        return 1;
    }

    @RequestMapping(value = {"personal"},method = {RequestMethod.POST, RequestMethod.GET})
    public String personal() {
        Map<String, Object> model = new HashMap();
        if(this.getCurUser()==null||this.getCurUser().getUserType()!= Constant.CUSTOMER_USER){
            return this.createFailJsonResp("请先登录");
        } else {
            JSONObject ret = new JSONObject();
            String busiDesc = "用户查看个人信息";
            Context context = this.getContext();
            try {
                CustomerEntity entity = this.service.get(this.getCurUser().getId(), context);
                if (entity == null) {
                    throw new Exception(busiDesc + "，不存在或已被删除");
                }
                entity.setPassword(null);
                ret.put("data", entity);
                this.recordSysLog(this.request, busiDesc + " 【成功】");
            } catch (Exception var8) {
                this.doException(this.request, busiDesc, model, var8);
                Object msg = model.get("message_info");
                return this.createFailJsonResp(msg == null ? "系统异常" : msg.toString());
            }

            this.init(model, context);
            ret.put("code", 1);
            ret.put("msg", model.remove("message_info"));
            ret.put("dict", model.remove("dict"));
            return ret.toJSONString();
        }
    }

    @RequestMapping(value = {"personal/update"},method = {RequestMethod.POST})
    public String updateByPersonal(@RequestBody CustomerEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        IUser user = this.getCurUser();
        if(user == null||this.getCurUser().getUserType()!= Constant.CUSTOMER_USER){
            return this.createFailJsonResp("请先登录");
        }
        String busiDesc = "客户修改密码";
        int code = 1;
        try {
            entity.setUpdateTime(new Date());
            entity.setId(user.getId());
            entity.setUpdateUserId(user.getId());
            entity.setUpdateUser(user.getLoginName());
            entity.setUpdateUserName(user.getRealName());
            entity.setUpdateUserDeptId(user.getDeptId());
            entity.setUpdateUserDeptName(user.getDeptName());
            this.service.updateByPersonal(entity, context);
            model.put("id", entity.getId());
            code = this.saveAfter(entity, model, context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    @RequestMapping(value = {"personal/change/password"},method = {RequestMethod.POST})
    public String changePassword(@RequestBody CustomerEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        IUser user = this.getCurUser();
        if(user == null||this.getCurUser().getUserType()!= Constant.CUSTOMER_USER){
            return this.createFailJsonResp("请先登录");
        }
        String busiDesc = "客户修改密码";
        int code = 1;
        try {
            entity.setUpdateTime(new Date());
            entity.setId(user.getId());
            entity.setUpdateUserId(user.getId());
            entity.setUpdateUser(user.getLoginName());
            entity.setUpdateUserName(user.getRealName());
            entity.setUpdateUserDeptId(user.getDeptId());
            entity.setUpdateUserDeptName(user.getDeptName());
            this.service.changePassword(entity, context);
            model.put("id", entity.getId());
            code = this.saveAfter(entity, model, context);
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

    @RequestMapping(value = {"admin/change/password"},method = {RequestMethod.POST})
    public String changePasswordByAdmin(@RequestBody CustomerEntity entity) {
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        IUser user = this.getCurUser();
        if(user == null){
            return this.createFailJsonResp("请先登录");
        }
        String busiDesc = "管理员修改密码";
        int code = 1;
        try {
            this.service.changePasswordByAdmin(entity, context);
            model.put("id", entity.getId());
            model.put("entity", entity);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】 [id:" + entity.getId() + "]");
        } catch (Exception var7) {
            this.doException(this.request, busiDesc, model, var7);
            model.put("entity", entity);
            this.init(model, context);
            code = this.saveException(entity, model, context, var7);
        }

        this.init(model, context);
        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }


    @Override
    @UnAuth
    public void exportExcel(@RequestBody CustomerEntity query) {
        Context context = this.getContext();
        String busiDesc = "导出" + this.getModuleDesc();

        try {
            this.doExportBefore(context, query);
            String name = StringUtils.trim(this.moduleDesc);
            if (StringUtils.isEmpty(name)) {
                name = System.currentTimeMillis() + "";
            }

            String fileName = name + ".xlsx";
            PageInfo pageInfo = new PageInfo(-1);
            List<CustomerEntityExt> list = this.service.findExt(query, pageInfo, this.getContext()).getList();
            Class<CustomerEntityExt> tClass = ReflectUtils.getClassGenricType(this.getClass(), 1);
            ExcelUtil<CustomerEntityExt, Long> util = new ExcelUtil(tClass);
            byte[] data = util.exportExcel(list, name);
            this.responseStream(this.response, data, fileName);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var11) {
            this.doException(this.request, "导出异常", new HashMap(), var11);
        }

    }
}