package com.mortals.xhx.module.question.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.EvaluateLevelEnum;
import com.mortals.xhx.module.question.dao.QuestionDao;
import com.mortals.xhx.module.question.model.QuestionEntity;
import com.mortals.xhx.module.question.model.QuestionQuery;
import com.mortals.xhx.module.question.model.QuestionRecordEntity;
import com.mortals.xhx.module.question.service.QuestionRecordService;
import com.mortals.xhx.module.question.service.QuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
* QuestionService
* 问答信息 service实现
*
* @author zxfei
* @date 2022-06-28
*/
@Service("questionService")
public class QuestionServiceImpl extends AbstractCRUDServiceImpl<QuestionDao, QuestionEntity, Long> implements QuestionService {

    @Autowired
    private QuestionRecordService questionRecordService;

    @Override
    protected QuestionEntity findBefore(QuestionEntity entity, PageInfo pageInfo, Context context) throws AppException {
        QuestionQuery query = new QuestionQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            QuestionQuery condition1 = new QuestionQuery();
            QuestionQuery condition2 = new QuestionQuery();
            condition1.setContactTelphone(condition.toString());
            condition2.setCustName(condition.toString());
            List<QuestionQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            query.setOrConditionList(orConditionList);
        }
        if(StringUtils.isNotEmpty(entity.getStartTime())){
            query.setBeginTimeStart(entity.getStartTime());
        }
        if(StringUtils.isNotEmpty(entity.getEndTime())){
            query.setBeginTimeEnd(entity.getEndTime());
        }
        return query;
    }

    @Override
    protected void findAfter(QuestionEntity params, PageInfo pageInfo, Context context, List<QuestionEntity> list) throws AppException {
        if(CollectionUtils.isNotEmpty(list)){
            list.stream().forEach(item->{
                QuestionRecordEntity query = new QuestionRecordEntity();
                query.setQuestionId(item.getId());
                List<QuestionRecordEntity> recordEntities = questionRecordService.find(query);
                item.setSumQA(CollectionUtils.isNotEmpty(list)?recordEntities.size():0);
            });
        }
    }

    @Override
    public QuestionEntity evaluate(QuestionEntity entity, Context context) throws AppException {
        if(entity.getId()==null){
            throw new AppException("问答不存在,请从新选择");
        }
        QuestionEntity question = this.get(entity.getId());
        if(question==null){
            throw new AppException("问答不存在,请从新选择");
        }
        if(entity.getEvaluate()==null){
            entity.setEvaluate(EvaluateLevelEnum.LEVEL_5.getValue());
        }
        QuestionEntity update = new QuestionEntity();
        update.setId(entity.getId());
        update.setEvaluate(entity.getEvaluate());
        return this.update(entity,context);
    }
}